package com.biganto.visual.roompark.data.repository.db.requrey.model

import com.biganto.visual.roompark.data.repository.api.biganto.IBigantoMobileApi.Companion.BASE_URL
import com.biganto.visual.roompark.data.repository.api.biganto.raw.LiteTourMetaRaw
import com.biganto.visual.roompark.data.repository.api.biganto.raw.TourFileRaw
import com.biganto.visual.roompark.data.repository.db.requrey.RevisionString
import com.biganto.visual.roompark.data.repository.db.requrey.utils.RevisionStringConverter
import com.biganto.visual.roompark.data.repository.file.FileModule
import io.requery.*
import java.io.File
import java.math.BigInteger
import java.security.MessageDigest

/**
 * Created by Vladislav Bogdashkin on 15.06.2018.
 */

val domainPredicthMatcher = "^(http|https)://".toRegex()
fun toInitialName(fullPath: String) = String.format(
       "%032x"
       , BigInteger(
              1
              , MessageDigest.getInstance("MD5").digest(fullPath.toByteArray(Charsets.UTF_8))
       )
)

@Entity
interface File :Persistable {
//       @get:Key
//       var id: String
//       @get:Convert(URIConverter::class)
       @get:Key
       @get:Convert(RevisionStringConverter::class)
       val uri: RevisionString
       val totalSize: Long
       val downloadedSize: Long
       @get:Nullable
       val isDownloaded: Boolean
       @get:Nullable
       val destination: String
}


fun fromRaw(raw: TourFileRaw, meta: LiteTourMetaRaw):FileEntity {

       val entity = FileEntity()
       entity.setDownloaded(false)
       entity.setDownloadedSize(0L)
       entity.setTotalSize(raw.size)

       var fileUri=raw.url

       domainPredicthMatcher.containsMatchIn(raw.url)
       val bui = raw.url.indexOf(meta.baseurl)
       if (bui>=0) fileUri = toInitialName(meta.baseurl)
              .plus(File.separator)
              .plus(raw.url.removeRange(0,bui+meta.baseurl.length))


       val tui = raw.url.indexOf(meta.tour_baseurl)
       if (tui>=0) fileUri = toInitialName(meta.tour_baseurl)
              .plus(File.separator)
              .plus(raw.url.removeRange(0,tui+meta.tour_baseurl.length))


       entity.setUri(RevisionString(FileModule.assetsLocalPath(fileUri)))

       entity.setDestination(
              if (domainPredicthMatcher.containsMatchIn(raw.url)) raw.url
              else BASE_URL.plus(raw.url)
       )

       return entity
}


fun fromRaw(raw: List<TourFileRaw>, meta: LiteTourMetaRaw): List<FileEntity> =
       List(raw.size) { index-> fromRaw(raw[index],meta) }
